#include "testApp.h"


//--------------------------------------------------------------
void testApp::setup(){	
	
	ofSetVerticalSync(true);
	ofSetFrameRate(60);
	ofBackground( 255, 255, 255 );
	
	ofSetCircleResolution( 100 );
	
	innerRadius = 150;
	outerRadius = 200;
	
	numberTries = 10;
	currentTry	= -1;
	
	numberBorder = 60;
	
	
	for (int i = 0; i < 200; i++){
		particle myParticle;
		//myParticle.setInitialCondition(ofRandom(450,550),ofRandom(0,ofGetHeight()),0,0);
		myParticle.setInitialCondition(ofGetWidth()/2 + ofRandom(-20, 20), ofGetHeight()/2 + ofRandom(-20, 20),0,0);
		particles.push_back(myParticle);
	}
	
	for ( int i = 0; i < numberTries; i++ ){
		particle myParticle;
		float angle = i * 2*PI/numberTries;
		myParticle.setInitialCondition( ofGetWidth()/2 + outerRadius * cos( angle ), ofGetHeight()/2 + outerRadius * sin( angle ), 0, 0 );
		tries.push_back( myParticle );
	}
	
	for ( int i = 0; i < numberBorder; i++ ){
		particle myParticle;
		float angle = i * 2*PI/numberBorder;
		myParticle.setInitialCondition( ofGetWidth()/2 + innerRadius * cos( angle ), ofGetHeight()/2 + innerRadius * sin( angle ), 0, 0 );
		border.push_back( myParticle );
	}
	
	VF.setupField(60,40,ofGetWidth(), ofGetHeight());
	VF.randomizeField(2.0);
	bParticlesRepel = true;
	bFade = false;
}

//--------------------------------------------------------------
void testApp::update(){

	// on every frame 
	// we reset the forces
	// add in any forces on the particle
	// perfom damping and
	// then update
	
	
	for (int i = 0; i < particles.size(); i++){
		particles[i].resetForce();
		
		// get the force from the vector field: 
		ofxVec2f frc;
		frc = VF.getForceFromPos(particles[i].pos.x, particles[i].pos.y);
		particles[i].addForce(frc.x, frc.y);
		
		if (bParticlesRepel == true){
			for (int j = 0; j < i; j++){
				particles[i].addRepulsionForce(particles[j], 50, 0.25);
			}
		}
		
		for ( int j = 0; j < numberBorder; j++ ){
			particles[i].addRepulsionForce(border[j], 30, 5 );
		}
		
		particles[i].addRepulsionForce(tries[currentTry],300, 3);
		//particles[i].addAttractionForce(ofGetWidth()/2, ofGetHeight()/2, 250, 1);
		
		//particles[i].addForce(0,0.4f);
		
		
		//particles[i].addForce( -(particles[i].pos.x-mouseX)*.005f, -(particles[i].pos.y-mouseY)*.005f);
		//particles[i].addAttractionForce( mouseX, mouseY, 300, 1 );
		particles[i].addAttractionForce( ofGetWidth()/2, ofGetHeight()/2, 250, 1 );
		
		//particles[i].addForce( -(ofGetWidth()/2-mouseX)*.001f, 0);
		
		particles[i].addDampingForce();
		particles[i].update();
		
		
		if ( ofDist( particles[ i ].pos.x, particles[ i ].pos.y, ofGetWidth()/2, ofGetHeight()/2 ) > innerRadius ){
			particles[ i ].vel *= -1;
		}
		
		/*if (particles[i].pos.y > ofGetHeight()){
			
			particles[ i ].pos.x
			particles[i].pos.x = ofRandom(450,550);
			particles[i].pos.y = 0;

		}*/
	}
	
	float angle = currentTry * 2 * PI/numberTries;
	tries[ currentTry ].resetForce();
	tries[ currentTry ].addRepulsionForce( ofGetWidth()/2 + 2*outerRadius * cos( angle ), ofGetHeight()/2 + 2*outerRadius * sin( angle ), 300, .75 );
	//tries[ currentTry ].addDampingForce();
	tries[ currentTry ].update();
	
	if (bFade == true) VF.fadeField(0.99f);

}

//--------------------------------------------------------------
void testApp::draw(){
	
	ofEnableAlphaBlending();
	ofSetColor(0,130,130, 200);
	//VF.draw();
	
	ofSetColor( 100, 100, 100 );
	ofNoFill();
	ofCircle( ofGetWidth()/2, ofGetHeight()/2, outerRadius );
	ofCircle( ofGetWidth()/2, ofGetHeight()/2, innerRadius );
	
	ofSetColor( 255, 0, 150 );
	ofFill();
	for ( int i = 0; i < numberTries; i++ ){
		ofCircle( tries[ i ].pos.x, tries[ i ].pos.y, 10 );
	}
	
	ofSetColor(0, 255, 0, 100);
	for ( int i = 0; i < numberBorder; i++ ){
		ofCircle( border[ i ].pos.x, border[ i ].pos.y, 10 );
	}
	
	ofFill();
	ofSetColor(0x222222);
	
	for (int i = 0; i < particles.size(); i++){
		particles[i].draw();
	}
	
	ofSetColor(0,130,130, 200);
	ofRect(30,30,300,75);
	ofSetColor(255,255,255, 230);
	ofDrawBitmapString("space to clear\ntoggle fade 'f'\ntoggle repel 'r'", 50, 50);
	
	
	ofSetColor(255,255,130, 230);
	
}

//--------------------------------------------------------------
void testApp::keyPressed  (int key){ 
	
	if (key == ' '){
		VF.clear();
	} else if (key == 'f'){
		bFade = !bFade;
	} else if (key == 'r'){
		bParticlesRepel = !bParticlesRepel;
	} else if (key == 'k'){
		currentTry++ ;
		printf("K");
	}


}	

//--------------------------------------------------------------
void testApp::keyReleased  (int key){ 
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	if (button == 0) {
		VF.addOutwardCircle((float)x, (float)y, 100, 0.29f);
	} else {
		
		particles.erase(particles.begin());
		particle myParticle;
		myParticle.setInitialCondition(x,y,0,0);
		particles.push_back(myParticle);
		
		
	}
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
}

//--------------------------------------------------------------
void testApp::mouseReleased(){
}
